@@ -66,14 +66,16 @@ def set_logdir(dirpath):
     G.LOGGER.set_logfile(os.path.join(ST.LOG_DIR, ST.LOG_FILE))
 
 
-def log(tag, data, in_stack=True):
-    if G.LOGGER:
-        G.LOGGER.log(tag, data, in_stack)
+def log(message, traceback=""):
+    """
+    Insert user log, will be displayed in Html report.
 
-
-def log_in_func(data):
+    :param message: log message
+    :param traceback: log traceback if exists, use traceback.format_exc to get best format
+    :return: None
+    """
     if G.LOGGER:
-        G.LOGGER.extra_log.update(data)
+        G.LOGGER.log("info", {"name": message, "traceback": traceback}, 0)
 
 
 def logwrap(f):
@@ -97,41 +99,23 @@ def using(path):
     G.BASEDIR.append(path)
 
 
-NATIVE_PFS = ["Android", "Windows", "IOS"]
-
-
 def import_device_cls(platform):
     """lazy import device class"""
     platform = platform.lower()
     if platform in G.CUSTOM_DEVICES:
         cls = G.CUSTOM_DEVICES[platform]
     elif platform == "android":
-        from .android import Android as cls
+        from airtest.core.android.android import Android as cls
     elif platform == "windows":
-        from .win import Windows as cls
+        from airtest.core.win.win import Windows as cls
     elif platform == "ios":
-        from .ios import IOS as cls
+        from airtest.core.ios.ios import IOS as cls
     elif platform == "linux":
-        from .linux import Linux as cls
+        from airtest.core.linux.linux import Linux as cls
     else:
         raise RuntimeError("Unknown platform: %s" % platform)
     return cls
 
 
-def on_platform(platforms):
-    def decorator(f):
-        @functools.wraps(f)
-        def wrapper(*args, **kwargs):
-            if G.DEVICE is None:
-                raise RuntimeError("Device not connected yet.")
-            pf = device_platform()
-            if pf in NATIVE_PFS and pf not in platforms:
-                raise NotImplementedError("Method not implememted on {}. required {}.".format(pf, platforms))
-            r = f(*args, **kwargs)
-            return r
-        return wrapper
-    return decorator
-
-
 def delay_after_operation():
     time.sleep(ST.OPDELAY)
